#include <gtk/gtk.h>
#include <glib/gstdio.h>
#include <gdk/gdkkeysyms.h>


#include <gmpc/plugin.h>
#include <gmpc/playlist3-messages.h>
#include <libmpd/libmpd.h>
#include <config.h>
#include "favorites.h"

#define DEFAULT_MARKUP_BROWSER 	"[%name%: &[%artist% - ]%title%]|%name%|[%artist% - ]%title%|%shortfile%|"

enum {
  FAV_COUNT,
  FAV_PATH,
  FAV_TITLE,
  FAV_NUM_TOTAL
};

static GtkTreeRowReference *favorites_browser_ref = NULL;
static GtkWidget *favorites_browser = NULL;


GKeyFile *fav_list = NULL;

GtkListStore *favorites_list_store = NULL;
/**
 * Functions from gmpc 
 */
int connection_get_port();
char *connection_get_hostname();

/**
 * Enable/Disable plugin
 */
static int favorites_get_enabled(void)
{
	return cfg_get_single_value_as_int_with_default(config, FAV_CONFIG, "enable", TRUE);
}
static void favorites_set_enabled(int enabled)
{
	cfg_set_single_value_as_int(config, FAV_CONFIG, "enable", enabled);
};


/**
 * Browser extention 
 */
gmpcPlBrowserPlugin favorites_gbp = {
  favorites_browser_add,   /* Add */
  favorites_browser_selected,   /* Selected */
  favorites_browser_unselected,   /* Unselected */
  favorites_browser_changed,   /* Changed */
  NULL,
  favorites_browser_cat_menu,   /* */
  NULL,   /* cat key press */
  favorites_go_menu, 
  favorites_key_press 
};

/**
 * Main plugin 
 */
int plugin_api_version = PLUGIN_API_VERSION;

gmpcPlugin plugin = {
  .name                         = "Favorites Plugin",
  .version                      = {PLUGIN_MAJOR_VERSION,PLUGIN_MINOR_VERSION,PLUGIN_MICRO_VERSION},
  .plugin_type                  = GMPC_PLUGIN_PL_BROWSER, 
  .init                         = favorites_init, 
  .destroy                      = favorites_destroy,
  .browser                      = &favorites_gbp,        
  .mpd_connection_changed       = favorites_conn_changed,
  .get_enabled                  = favorites_get_enabled,
  .set_enabled                  = favorites_set_enabled 
};

/**
 * Init plugin
 */
void favorites_init(void)
{
    gchar *path = gmpc_get_user_path("favorites");
    /**
     * Check if favorites config dir exists, otherwise create it.
     */
    if(!g_file_test(path, G_FILE_TEST_EXISTS | G_FILE_TEST_IS_DIR))
    {
        g_mkdir(path, 0700);
    }
    g_free(path);
}
/**
 * Destroy Plugin
 */
void favorites_destroy(void)
{
  /* Cleanup open config */
  favorites_conn_changed(connection, FALSE, NULL);
  if(favorites_browser)
  {
    gtk_widget_destroy(favorites_browser);
    favorites_browser = NULL;
  }
}


/**
 * Connection changed
 */
void favorites_conn_changed(MpdObj *mi, int connect, void *userdata)
{
  if(fav_list)
  {
    g_key_file_free(fav_list);
    fav_list = NULL;
  }
  /* If we are connected open list */
  if(connect)
  {
    gchar *path = g_strdup_printf("%s%c.gmpc%cfavorites%c%s-%i.cfg",g_get_home_dir(), G_DIR_SEPARATOR, G_DIR_SEPARATOR,G_DIR_SEPARATOR,
        connection_get_hostname(), connection_get_port() ); 

    fav_list = g_key_file_new();
    if(g_file_test(path, G_FILE_TEST_EXISTS|G_FILE_TEST_IS_REGULAR))
    {
      g_key_file_load_from_file(fav_list, path,G_KEY_FILE_NONE, NULL);
    }
    
    g_free(path);

    favorites_browser_fill_list();
  }
}
/** 
 * Browser
 */
void favorites_browser_add(GtkWidget *cat_tree)
{
  GtkTreeStore *pl3_tree = playlist3_get_category_tree_store();
	GtkTreePath *path = NULL;
	GtkTreeIter iter;
  /* Check if enabled */
  if(!cfg_get_single_value_as_int_with_default(config, FAV_CONFIG, "enable", TRUE)) return;

	gtk_tree_store_append(pl3_tree, &iter, NULL);
	gtk_tree_store_set(pl3_tree, &iter, 
			PL3_CAT_TYPE, plugin.id,
			PL3_CAT_TITLE, "Favorites Browser",
			PL3_CAT_INT_ID, "",
			PL3_CAT_ICON_ID, "gtk-yes",
			PL3_CAT_PROC, TRUE,
			PL3_CAT_ICON_SIZE,GTK_ICON_SIZE_DND,-1);
	/**
	 * Clean up old row reference if it exists
	 */
	if (favorites_browser_ref)
	{
		gtk_tree_row_reference_free(favorites_browser_ref);
		favorites_browser_ref = NULL;
	}
	/**
	 * create row reference
	 */
	path = gtk_tree_model_get_path(GTK_TREE_MODEL(playlist3_get_category_tree_store()), &iter);
	if (path)
	{
		favorites_browser_ref = gtk_tree_row_reference_new(GTK_TREE_MODEL(playlist3_get_category_tree_store()), path);
		gtk_tree_path_free(path);
	}
}
void favorites_browser_row_activated(GtkTreeView *tree, GtkTreePath *path, GtkTreeViewColumn *col, gpointer data)
{
  GtkTreeModel *model = gtk_tree_view_get_model(tree);
  GtkTreeIter iter;
  if(gtk_tree_model_get_iter(model, &iter, path))
  {
    gchar *spath = NULL;
    gtk_tree_model_get(model, &iter, FAV_PATH, &spath, -1);
    if(spath)
    {
        play_path(spath);
        g_free(spath);
    }
  }
}

static void favorites_browser_init()
{
  GtkCellRenderer *renderer = NULL;
  GtkTreeViewColumn *column = NULL;
  GtkWidget *tree = NULL;
  GtkWidget *sw = NULL;
  GtkWidget *label;
  /* */
  favorites_browser = gtk_vbox_new(FALSE,0);

  /* Scroll window */
  sw = gtk_scrolled_window_new(NULL, NULL);
  gtk_scrolled_window_set_policy(GTK_SCROLLED_WINDOW(sw), GTK_POLICY_AUTOMATIC, GTK_POLICY_AUTOMATIC);
  gtk_scrolled_window_set_shadow_type(GTK_SCROLLED_WINDOW(sw), GTK_SHADOW_ETCHED_IN);
  gtk_box_pack_start(GTK_BOX(favorites_browser), sw, TRUE,TRUE,6);


  label = gtk_label_new("To favor the current playing song, press Ctrl-Enter");
  gtk_misc_set_alignment(GTK_MISC(label), 0,0.5);
  gtk_box_pack_start(GTK_BOX(favorites_browser), label, FALSE,TRUE,6);

  /* treeview */
  tree = gtk_tree_view_new();
  gtk_tree_view_set_headers_visible(GTK_TREE_VIEW(tree), TRUE);
  gtk_container_add(GTK_CONTAINER(sw), tree);

  g_signal_connect(G_OBJECT(tree), "row-activated", G_CALLBACK(favorites_browser_row_activated), NULL);
  /* Model */
  favorites_list_store = gtk_list_store_new(FAV_NUM_TOTAL,G_TYPE_INT, G_TYPE_STRING, G_TYPE_STRING);

  renderer = gtk_cell_renderer_text_new();
  column = gtk_tree_view_column_new_with_attributes("Times Favored", renderer,"text", FAV_COUNT,NULL);
  gtk_tree_view_append_column(GTK_TREE_VIEW(tree), column);

  renderer = gtk_cell_renderer_text_new();
  column = gtk_tree_view_column_new_with_attributes("Favorites", renderer,"text", FAV_TITLE,NULL);
  gtk_tree_view_append_column(GTK_TREE_VIEW(tree), column);

  gtk_tree_view_set_model(GTK_TREE_VIEW(tree), GTK_TREE_MODEL(favorites_list_store));
  g_object_ref(favorites_browser);

  favorites_browser_fill_list();
}

void favorites_browser_selected(GtkWidget *container)
{
  if(!favorites_browser) {
    favorites_browser_init();
  }
  gtk_container_add(GTK_CONTAINER(container), favorites_browser);
  gtk_widget_show_all(favorites_browser);
}

void favorites_browser_unselected(GtkWidget *container)
{
  gtk_container_remove(GTK_CONTAINER(container), favorites_browser);
}

void favorites_browser_changed(GtkWidget *tree, GtkTreeIter *iter)
{
  /* Do nothing */
}

void favorites_browser_fill_list(void)
{
  int changed =0;
  if(fav_list && favorites_list_store)
  {
    gchar **groups;
    int i;
    gtk_list_store_clear(favorites_list_store);

    groups = g_key_file_get_groups(fav_list, NULL);
    for(i=0;groups[i];i++)
    {
      int count = 0;
      GtkTreeIter iter;
      gchar *text = NULL;
      gtk_list_store_append(favorites_list_store, &iter);
      text = g_key_file_get_string(fav_list, groups[i], "title", NULL);
      if(!text)
      {
        mpd_Song *song = mpd_database_get_fileinfo(connection, groups[i]);
        if(song)
        {
          gchar *markdata = cfg_get_single_value_as_string_with_default(config, "playlist", "browser_markup",DEFAULT_MARKUP_BROWSER);
          text = g_malloc(1024*sizeof(*text));
          mpd_song_markup(text, 1024, markdata, song);
          g_key_file_set_string(fav_list, groups[i], "title", text);
          changed = 1;
          g_free(markdata);
        }
        else text = g_strdup("Song not found");
      }
      count = g_key_file_get_integer(fav_list, groups[i], "added",NULL);
      gtk_list_store_set(favorites_list_store, &iter, FAV_PATH, groups[i], FAV_TITLE, text,FAV_COUNT,count,-1);
      g_free(text);


    }
    g_strfreev(groups);
  }
  if(changed)
    favorites_save();
}

int favorites_browser_cat_menu(GtkWidget *menu, int type, GtkWidget *tree, GdkEventButton *event)
{
  if(type == plugin.id)
  {
    GtkWidget *item;
    /** */
    item = gtk_image_menu_item_new_with_label("Load favorites playlist");
    gtk_image_menu_item_set_image(GTK_IMAGE_MENU_ITEM(item), gtk_image_new_from_stock(GTK_STOCK_NEW, GTK_ICON_SIZE_MENU));
    g_signal_connect(G_OBJECT(item), "activate", G_CALLBACK(favorites_create_playlist), NULL);
    /* add to menu */
    gtk_menu_shell_append(GTK_MENU_SHELL(menu), item);
    /** */
    item = gtk_image_menu_item_new_with_label("Clear favorites");
    gtk_image_menu_item_set_image(GTK_IMAGE_MENU_ITEM(item), gtk_image_new_from_stock(GTK_STOCK_CLEAR, GTK_ICON_SIZE_MENU));
    g_signal_connect(G_OBJECT(item), "activate", G_CALLBACK(favorites_clear), NULL);
    /* add to menu */
    gtk_menu_shell_append(GTK_MENU_SHELL(menu), item);


    return 2;
  }
  return 0;
}

/**
 * MW Intergration
 */
int favorites_go_menu(GtkWidget *menu)
{
  GtkWidget *item;
  /** */
  item = gtk_image_menu_item_new_with_label("Add Current Song to favorites");
  gtk_image_menu_item_set_image(GTK_IMAGE_MENU_ITEM(item), gtk_image_new_from_stock(GTK_STOCK_YES, GTK_ICON_SIZE_MENU));
  g_signal_connect(G_OBJECT(item), "activate", G_CALLBACK(favorites_add_current_song), NULL);

  /* add to menu */
  gtk_menu_shell_append(GTK_MENU_SHELL(menu), item);
  return 1;
}

int favorites_key_press(GtkWidget *mw, GdkEventKey *event, int type)
{
/* FIXME */
  if(event->keyval == GDK_Return && event->state&GDK_CONTROL_MASK)
  {
    favorites_add_current_song();
    return TRUE;
  }

  return FALSE;
}

void favorites_save(void)
{
  gchar *data;
  gsize size;
  gchar *path = g_strdup_printf("%s%c.gmpc%cfavorites%c%s-%i.cfg",g_get_home_dir(), G_DIR_SEPARATOR, G_DIR_SEPARATOR,G_DIR_SEPARATOR,
      connection_get_hostname(), connection_get_port() ); 
  if(fav_list)
  {
    data = g_key_file_to_data(fav_list, &size, NULL);
    g_file_set_contents(path, data, size, NULL);
  }
  g_free(path);
}

void favorites_add_current_song(void)
{
  mpd_Song *song = mpd_playlist_get_current_song(connection);
  if(song && fav_list) 
  {
      gchar *basename = g_path_get_basename(song->file);
      gchar *message = g_strdup_printf("Added '%s' to the favorites list", basename);
      playlist3_show_error_message(message, ERROR_INFO);
      g_free(basename);
      if(g_key_file_has_group(fav_list, song->file))
      {


          int added = g_key_file_get_integer(fav_list, song->file, "added",NULL);
          g_key_file_set_integer(fav_list, song->file, "added",added+1);


      }
      else /* no key so add with count 1 */
      {
          g_key_file_set_integer(fav_list, song->file, "added",1);
      }
      favorites_save();
      favorites_browser_fill_list();
  }
  else {
    printf("no song to add\n");
  }
}

void favorites_create_playlist(void)
{
    int i=0;
    gchar **groups = g_key_file_get_groups(fav_list, NULL);
    mpd_playlist_clear(connection);
    for(i=0;groups[i];i++)
    {
      mpd_playlist_queue_add(connection, groups[i]);
    }
    mpd_playlist_queue_commit(connection);
    mpd_player_play(connection);
    g_strfreev(groups);
}

void favorites_clear(void)
{
  if(fav_list) {
    /* completely clear the old one, and create a new one */
    g_key_file_free(fav_list);
    fav_list = g_key_file_new();

    favorites_save();
    favorites_browser_fill_list();
  }
}

